//========================================================================
// Verilog_LFSR2
//========================================================================
// Updated LFSR based on one used in link below for stochastic chemical
// reaction simulation.
// Link: http://people.ece.cornell.edu/land/courses/ece5760/Chemical_
// Simulation/Two_reaction_per_step/VGA_320x240_chem_sim_2_MM_rand127bit.v

`ifndef MENG_PROJECT_LFSR2
`define MENG_PROJECT_LFSR2

module Verilog_LFSR2
(
	input  wire [3:0]   state_in,
	input               clk,
	input               reset,
	input  wire [127:0] seed,
	output wire [15:0]  rand_out
);
	// Individual shift registers
	reg [8:1] sr1, sr2, sr3, sr4, sr5, sr6, sr7, sr8, sr9, sr10, sr11,
			  sr12, sr13, sr14, sr15, sr16;
	
	// State names
		// State for start of module
	parameter mod_start = 4'h0;
	
	// Generate random number
	assign rand_out = {sr1[7],sr2[7],sr3[7],sr4[7],sr5[7],sr6[7],sr7[7],
					   sr8[7],sr9[7],sr10[7],sr11[7],sr12[7],sr13[7],
					   sr14[7],sr15[7],sr16[7]};
	
	always @(posedge clk)
	begin
		if(reset)
		begin
			// Initialize RNG
			sr1  <= seed[8:1]    ;
			sr2  <= seed[16:9]   ;
			sr3  <= seed[24:17]  ;
			sr4  <= seed[32:25]  ;
			sr5  <= seed[40:33]  ;
			sr6  <= seed[48:41]  ;
			sr7  <= seed[56:49]  ;
			sr8  <= seed[64:57]  ;
			sr9  <= seed[72:65]  ;
			sr10 <= seed[80:73]  ;
			sr11 <= seed[88:81]  ;
			sr12 <= seed[96:89]  ;
			sr13 <= seed[104:96] ;
			sr14 <= seed[112:105];
			sr15 <= seed[120:113];
			sr16 <= {1'b0,seed[127:121]} ;
		end
		else
		// Update all shift registers in parallel
		begin
			// if(state_in == mod_start)
			// begin
			sr1  <= {sr1[7:1],  sr1[7]^sr16[7]} ;
			sr2  <= {sr2[7:1],  sr2[7]^sr1[8]}  ;
			sr3  <= {sr3[7:1],  sr3[7]^sr2[8]}  ;
			sr4  <= {sr4[7:1],  sr4[7]^sr3[8]}  ;
			sr5  <= {sr5[7:1],  sr5[7]^sr4[8]}  ;
			sr6  <= {sr6[7:1],  sr6[7]^sr5[8]}  ;
			sr7  <= {sr7[7:1],  sr7[7]^sr6[8]}  ;
			sr8  <= {sr8[7:1],  sr8[7]^sr7[8]}  ;
			sr9  <= {sr9[7:1],  sr9[7]^sr8[8]}  ;
			sr10 <= {sr10[7:1], sr10[7]^sr9[8]} ;
			sr11 <= {sr11[7:1], sr11[7]^sr10[8]};
			sr12 <= {sr12[7:1], sr12[7]^sr11[8]};
			sr13 <= {sr13[7:1], sr13[7]^sr12[8]};
			sr14 <= {sr14[7:1], sr14[7]^sr13[8]};
			sr15 <= {sr15[7:1], sr15[7]^sr14[8]};
			sr16 <= {sr16[6:1], sr16[7]^sr15[8]};
			// end
		end
	end
endmodule

`endif /* MENG_PROJECT_LFSR */