//========================================================================
// Verilog_Register_Reset
//========================================================================
// Register with a parametrized number of bits. Triggers on positive edge
// of the clock.
//
// This version has a reset function that allows for setting register to
// known value.


`ifndef MENG_PROJECT_REG_RST
`define MENG_PROJECT_REG_RST

module Verilog_RegRst
#(
	// Parameter for number of bits and reset value
	parameter nbits = 16,
	parameter reset_val = 0
)(
	input					 clk,
	input					 reset,
	// Data signals
	input  		 [nbits-1:0] in,
	output 		 [nbits-1:0] out
);

	// Sequential logic
	reg         [nbits-1:0] temp_wire;
	always @( posedge clk ) 
	begin
		if ( reset ) begin
			temp_wire <= reset_val;
		end
		else begin
			temp_wire <= in;
		end
	end
	assign out = temp_wire;
endmodule


`endif /* MENG_PROJECT_REG_RST */